@@ -4,6 +4,7 @@ Welcome to `LibAFL`
 
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(feature = "RUSTC_IS_NIGHTLY", feature(specialization))]
+#![deny(rustdoc::broken_intra_doc_links)]
 
 #[macro_use]
 extern crate alloc;
@@ -43,7 +44,7 @@ use alloc::string::String;
 use core::fmt;
 
 #[cfg(feature = "std")]
-use std::{env::VarError, io, num::ParseIntError, string::FromUtf8Error};
+use std::{env::VarError, io, num::ParseIntError, num::TryFromIntError, string::FromUtf8Error};
 
 /// Main error struct for AFL
 #[derive(Debug)]
@@ -120,7 +121,7 @@ impl From<serde_json::Error> for Error {
     }
 }
 
-#[cfg(unix)]
+#[cfg(all(unix, feature = "std"))]
 impl From<nix::Error> for Error {
     fn from(err: nix::Error) -> Self {
         Self::Unknown(format!("{:?}", err))
@@ -156,6 +157,13 @@ impl From<ParseIntError> for Error {
     }
 }
 
+#[cfg(feature = "std")]
+impl From<TryFromIntError> for Error {
+    fn from(err: TryFromIntError) -> Self {
+        Self::IllegalState(format!("Expected conversion failed: {:?}", err))
+    }
+}
+
 // TODO: no_std test
 #[cfg(feature = "std")]
 #[cfg(test)]
