@@ -11,12 +11,19 @@ pub use mutational::{MutationalStage, StdMutationalStage};
 pub mod tracing;
 pub use tracing::{ShadowTracingStage, TracingStage};
 
-pub mod mopt;
-pub use mopt::*;
+pub mod calibrate;
+pub use calibrate::{CalibrationStage, PowerScheduleMetadata};
 
-//pub mod power;
-//pub use power::PowerMutationalStage;
+pub mod power;
 use crate::Error;
+pub use power::PowerMutationalStage;
+
+#[cfg(feature = "std")]
+pub mod concolic;
+#[cfg(feature = "std")]
+pub use concolic::ConcolicTracingStage;
+#[cfg(feature = "std")]
+pub use concolic::SimpleConcolicMutationalStage;
 
 /// A stage is one step in the fuzzing process.
 /// Multiple stages will be scheduled one by one for each input.
